/**
 * Clase Vector2D.
 * @author Ivn Hopenhayn
 */
public class Vector2D {
/**
 * Coordenada en eje x.
 */
        private double x;
/**
 * Coordenada en eje y.
 */
	private double y;  

	public Vector2D () {
		x = y = 0;
	}
    /** 
 * Constructor.  inicializa el vector2D 
 * @param x 	posicin en x
 * @param y 	posicin en y
 */

	public Vector2D (double x, double y) {
		this.x = x;
		this.y = y;
	}
        /**
 * Da la posicin en eje x  
 * @return la <tt>coordenada en x</tt>
 */
	public double getX(){
		return x;
	}
           /**
 * Da la posicin en eje y  
 * @return la <tt>coordenada en y</tt>
 */
	public double getY(){
		return y;
	}
/**
 * setea posicin.  
 * @param x   otorga posicin del eje x
 * @param y  otorga posicin del eje y 
 */
	public void set(double x, double y){ // to set a new position
		this.x=x;
		this.y=y;
	}
/**
 * Suma vectorial.  
 * @param v   vector a sumar, si <tt>v = null</tt> no hay efecto
 * @return suma de ambos vectores
 */
	public Vector2D plus(Vector2D v) {
		if (v==null) return new Vector2D(x,y);
		return new Vector2D(x+v.x, y+v.y);
	}
/**
 * Multiplica vector por una constante.  
 * @param scalar  constante que multiplica al vector
 * @return el vector multiplicado por el escalar
 */
	public Vector2D times(double scalar) {
		return new Vector2D(x*scalar, y*scalar);
	}
/**
 * Resta vectorial.  
 * @param v   vector a restar, if <tt>v = null</tt> no hay efecto
 * @return suma de ambos vectores
 */
	public Vector2D minus(Vector2D v) {
		if (v==null) return new Vector2D(x,y);
		return new Vector2D(x-v.x, y-v.y);
	}
/**
 * Multiplicacion de componentes del vector.  
 * @param v   vector a multiplicar por coordenadas
 * @return nuevo vector con componentes multiplicadas
 */

	public double dotProduct(Vector2D v) {
		return x*v.x + y*v.y;
	}
/**
 * Obtiene proyeccion sobre vector.  
 * @param v   vector sobre el cual se genera la proyeccin
 * @return vector proyectado
 */

	public Vector2D getProjectionOn(Vector2D v){
		double newModule = x*v.x + y*v.y;
		return v.times(newModule/(v.x*v.x + v.y*v.y));
	}
/**
 * Obtiene mdulo.  
 * @return mdulo del vector
 */
	public double module() {
		return Math.sqrt(x*x + y*y);
	}
/**
 * suma de cuadrados de componentes
 * @return el resultado de la suma
 */
	public double moduleSquared() {
		return x*x + y*y;
	}
/**
 * Vector unitario.  
 * @return un vector unitario 
 */
	public Vector2D unitary() {
		return times(1/module());
	}
/**
 * obtiene Descripcin del vector
 * @return string de las coordenadas del vector
 */
	public static String getDescription() {
		return "(x,y)";
	}
/**
 * Genera string de coordenadas.  
 * @return un string de la forma  Vector x,y
 */
	public String toString() {
		return "Vector "+x+","+y;
	}
}